@@ -29,17 +29,17 @@ from decimal import Context, Decimal, localcontext
 from inspect import isclass, isfunction
 from fractions import Fraction
 from functools import reduce
-from collections import Sequence
 
 import attr
 
+from hypothesis.types import RandomWithSeed
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.control import note, assume, reject, cleanup, \
     current_build_context
 from hypothesis._settings import note_deprecation
 from hypothesis.internal.cache import LRUReusedCache
 from hypothesis.searchstrategy import SearchStrategy, check_strategy
-from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
+from hypothesis.internal.compat import abc, gcd, ceil, floor, hrange, \
     string_types, get_type_hints, getfullargspec, typing_root_type, \
     implements_iterator
 from hypothesis.internal.floats import next_up, float_of, next_down, \
@@ -55,7 +55,7 @@ from hypothesis.internal.validation import check_type, try_convert, \
     check_valid_integer, check_valid_interval, check_valid_magnitude
 from hypothesis.searchstrategy.lazy import LazyStrategy
 from hypothesis.searchstrategy.misc import BoolStrategy, JustStrategy, \
-    RandomStrategy, SampledFromStrategy
+    SampledFromStrategy
 from hypothesis.searchstrategy.shared import SharedStrategy
 from hypothesis.searchstrategy.numbers import FloatStrategy, \
     BoundedIntStrategy, IntegersFromStrategy, WideRangeIntStrategy, \
@@ -88,7 +88,6 @@ except ImportError:
 
 if False:
     from types import ModuleType  # noqa
-    from random import Random  # noqa
     from typing import Any, Dict, Union, Sequence, Callable, Pattern  # noqa
     from typing import TypeVar, Tuple, List, Set, FrozenSet, overload  # noqa
     from typing import Type, Text, AnyStr, Optional  # noqa
@@ -291,7 +290,7 @@ def one_of(*args):
     strategies.
 
     This may be called with one iterable argument instead of multiple
-    strategy arguments. In which case ``one_of(x)`` and ``one_of(*x)`` are
+    strategy arguments, in which case ``one_of(x)`` and ``one_of(*x)`` are
     equivalent.
 
     Examples from this strategy will generally shrink to ones that come from
@@ -317,8 +316,8 @@ def one_of(*args):
 @defines_strategy_with_reusable_values
 def integers(min_value=None, max_value=None):
     # type: (Real, Real) -> SearchStrategy[int]
-    """Returns a strategy which generates integers (in Python 2 these may be
-    ints or longs).
+    """Returns a strategy which generates integers; in Python 2 these may be
+    ints or longs.
 
     If min_value is not None then all values will be >= min_value. If
     max_value is not None then all values will be <= max_value
@@ -334,6 +333,19 @@ def integers(min_value=None, max_value=None):
     min_int_value = None if min_value is None else ceil(min_value)
     max_int_value = None if max_value is None else floor(max_value)
 
+    if min_value != min_int_value:
+        note_deprecation(
+            'min_value=%r of type %r cannot be exactly represented as an '
+            'integer, which will be an error in a future version.  '
+            'Use %r instead.' % (min_value, type(min_value), min_int_value)
+        )
+    if max_value != max_int_value:
+        note_deprecation(
+            'max_value=%r of type %r cannot be exactly represented as an '
+            'integer, which will be an error in a future version.  '
+            'Use %r instead.' % (max_value, type(max_value), max_int_value)
+        )
+
     if min_int_value is not None and max_int_value is not None and \
             min_int_value > max_int_value:
         raise InvalidArgument('No integers between min_value=%r and '
@@ -368,7 +380,7 @@ def integers(min_value=None, max_value=None):
 @defines_strategy
 def booleans():
     # type: () -> SearchStrategy[bool]
-    """Returns a strategy which generates instances of bool.
+    """Returns a strategy which generates instances of :class:`python:bool`.
 
     Examples from this strategy will shrink towards False (i.e.
     shrinking will try to replace True with False where possible).
@@ -403,7 +415,7 @@ def floats(
     Passing ``width=32`` will still use the builtin 64-bit ``float`` class,
     but always for values which can be exactly represented as a 32-bit float.
     Half-precision floats (``width=16``) are only supported on Python 3.6, or
-    if Numpy is installed.
+    if :pypi:`Numpy` is installed.
 
     Examples from this strategy have a complicated and hard to explain
     shrinking behaviour, but it tries to improve "human readability". Finite
@@ -441,6 +453,21 @@ def floats(
         max_value = float_of(max_value, width)
         assert isinstance(max_value, float)
 
+    if min_value != min_arg:
+        note_deprecation(
+            'min_value=%r cannot be exactly represented as a float of width '
+            '%d, which will be an error in a future version. Use min_value=%r '
+            'instead.'
+            % (min_value, width, min_arg)
+        )
+    if max_value != max_arg:
+        note_deprecation(
+            'max_value=%r cannot be exactly represented as a float of width '
+            '%d, which will be an error in a future version. Use max_value=%r '
+            'instead'
+            % (max_value, width, max_arg)
+        )
+
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
     if min_value == float(u'-inf'):
         min_value = None
@@ -454,7 +481,7 @@ def floats(
         max_value = next_down(max_value, width)
         assert max_value < max_arg  # type: ignore
     if min_value is not None and max_value is not None and \
-            min_value > max_value:  # type: ignore
+            min_value > max_value:
         raise InvalidArgument(
             'There are no %s-bit floating-point values between min_value=%r '
             'and max_value=%r' % (width, min_arg, max_arg))
@@ -602,8 +629,8 @@ def sampled_from(elements):
 @defines_strategy
 def lists(
     elements=None,  # type: SearchStrategy[Ex]
-    min_size=None,  # type: int
-    average_size=None,  # type: int
+    min_size=0,  # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
     unique_by=None,  # type: Callable[..., Any]
     unique=False,  # type: bool
@@ -618,12 +645,13 @@ def lists(
     Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     If unique is True (or something that evaluates to True), we compare direct
-    object equality, as if unique_by was `lambda x: x`. This comparison only
+    object equality, as if unique_by was ``lambda x: x``. This comparison only
     works for hashable types.
 
     if unique_by is not None it must be a function returning a hashable type
     when given a value drawn from elements. The resulting list will satisfy the
-    condition that for i != j, unique_by(result[i]) != unique_by(result[j]).
+    condition that for ``i`` != ``j``, ``unique_by(result[i])`` !=
+    ``unique_by(result[j])``.
 
     Examples from this strategy shrink by trying to remove elements from the
     list, and by shrinking each individual element of the list.
@@ -671,8 +699,8 @@ def lists(
 @defines_strategy
 def sets(
     elements=None,  # type: SearchStrategy[Ex]
-    min_size=None,   # type: int
-    average_size=None,  # type: int
+    min_size=0,   # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
 ):
     # type: (...) -> SearchStrategy[Set[Ex]]
@@ -701,8 +729,8 @@ def sets(
 @defines_strategy
 def frozensets(
     elements=None,  # type: SearchStrategy[Ex]
-    min_size=None,   # type: int
-    average_size=None,  # type: int
+    min_size=0,   # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
 ):
     # type: (...) -> SearchStrategy[FrozenSet[Ex]]
@@ -738,7 +766,7 @@ class PrettyIter(object):
 
 
 @defines_strategy
-def iterables(elements=None, min_size=None, average_size=None, max_size=None,
+def iterables(elements=None, min_size=0, average_size=None, max_size=None,
               unique_by=None, unique=False):
     """This has the same behaviour as lists, but returns iterables instead.
 
@@ -788,8 +816,8 @@ def dictionaries(
     keys,  # type: SearchStrategy[Ex]
     values,  # type: SearchStrategy[T]
     dict_class=dict,  # type: type
-    min_size=None,  # type: int
-    average_size=None,  # type: int
+    min_size=0,  # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
 ):
     # type: (...) -> SearchStrategy[Dict[Ex, T]]
@@ -865,12 +893,12 @@ def characters(
     - If ``whitelist_characters`` is specified, then any additional characters
       in that list will also be produced.
     - If ``blacklist_characters`` is specified, then any characters in
-      that list will be not be produced. Any overlap between \
+      that list will be not be produced. Any overlap between
       ``whitelist_characters`` and ``blacklist_characters`` will raise an
       exception.
 
     The ``_codepoint`` arguments must be integers between zero and
-    :obj:`python:sys.max_unicode`.  The ``_characters`` arguments must be
+    :obj:`python:sys.maxunicode`.  The ``_characters`` arguments must be
     collections of length-one unicode strings, such as a unicode string.
 
     The ``_categories`` arguments must be used to specify either the
@@ -896,8 +924,11 @@ def characters(
             blacklist_categories is None,
             )):
         raise InvalidArgument(
-            'Passing only whitelist_characters=%r would have no effect. '
-            'Perhaps you want sampled_from() ?' % (whitelist_characters,))
+            'Nothing is excluded by other arguments, so passing only '
+            'whitelist_characters=%(chars)r would have no effect.  Also pass '
+            'whitelist_categories=(), or use sampled_from(%(chars)r) instead.'
+            % dict(chars=whitelist_characters)
+        )
     blacklist_characters = blacklist_characters or ''
     whitelist_characters = whitelist_characters or ''
     overlap = set(blacklist_characters).intersection(whitelist_characters)
@@ -935,42 +966,44 @@ def characters(
 @cacheable
 @defines_strategy_with_reusable_values
 def text(
-    alphabet=None,  # type: Union[Sequence[Text], SearchStrategy[Text]]
-    min_size=None,   # type: int
-    average_size=None,   # type: int
+    alphabet=characters(
+        blacklist_categories=('Cs',)
+    ),  # type: Union[Sequence[Text], SearchStrategy[Text]]
+    min_size=0,   # type: int
+    average_size=None,   # type: None
     max_size=None  # type: int
 ):
     # type: (...) -> SearchStrategy[Text]
     """Generates values of a unicode text type (unicode on python 2, str on
     python 3) with values drawn from alphabet, which should be an iterable of
-    length one strings or a strategy generating such. If it is None it will
-    default to generating the full unicode range (excluding surrogate
-    characters). If it is an empty collection this will only generate empty
-    strings.
+    length one strings or a strategy generating such strings.
+
+    The default alphabet strategy can generate the full unicode range but
+    excludes surrogate characters because they are invalid in the UTF-8
+    encoding.  You can use :func:`~hypothesis.strategies.characters` without
+    arguments to find surrogate-related bugs such as :bpo:`34454`.
 
     min_size and max_size have the usual interpretations.
+    Note that Python measures string length by counting codepoints: U+00C5
+    ``Å`` is a single character, while U+0041 U+030A ``Å`` is two - the ``A``,
+    and a combining ring above.
 
     The average_size argument is deprecated.  Internal upgrades since
     Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     Examples from this strategy shrink towards shorter strings, and with the
     characters in the text shrinking as per the alphabet strategy.
+    This strategy does not :func:`~python:unicodedata.normalize` examples,
+    so generated strings may be in any or none of the 'normal forms'.
     """
     check_valid_sizes(min_size, average_size, max_size)
     if alphabet is None:
+        note_deprecation('alphabet=None is deprecated; just omit the argument')
         char_strategy = characters(blacklist_categories=('Cs',))
-    elif not alphabet:
-        if (min_size or 0) > 0:
-            raise InvalidArgument(
-                'Invalid min_size %r > 0 for empty alphabet' % (
-                    min_size,
-                )
-            )
-        return just(u'')
     elif isinstance(alphabet, SearchStrategy):
         char_strategy = alphabet
     else:
-        if not isinstance(alphabet, Sequence):
+        if not isinstance(alphabet, abc.Sequence):
             note_deprecation(
                 'alphabet must be an ordered sequence, or tests may be '
                 'flaky and shrinking weaker, but a %r is not a type of '
@@ -995,6 +1028,8 @@ def text(
                 'will be an error in future:  %r' % (not_one_char,)
             )
         char_strategy = sampled_from(alphabet)
+    if (max_size == 0 or char_strategy.is_empty) and not min_size:
+        return just(u'')
     return StringStrategy(lists(
         char_strategy, min_size=min_size, max_size=max_size
     ))
@@ -1002,17 +1037,17 @@ def text(
 
 @cacheable
 @defines_strategy
-def from_regex(regex):
-    # type: (Union[AnyStr, Pattern[AnyStr]]) -> SearchStrategy[AnyStr]
-    """Generates strings that contain a match for the given regex (i.e. ones
-    for which :func:`re.search` will return a non-None result).
+def from_regex(regex, fullmatch=False):
+    # type: (Union[AnyStr, Pattern[AnyStr]], bool) -> SearchStrategy[AnyStr]
+    r"""Generates strings that contain a match for the given regex (i.e. ones
+    for which :func:`python:re.search` will return a non-None result).
 
     ``regex`` may be a pattern or :func:`compiled regex <python:re.compile>`.
     Both byte-strings and unicode strings are supported, and will generate
     examples of the same type.
 
-    You can use regex flags such as :const:`re.IGNORECASE`, :const:`re.DOTALL`
-    or :const:`re.UNICODE` to control generation. Flags can be passed either
+    You can use regex flags such as :obj:`python:re.IGNORECASE` or
+    :obj:`python:re.DOTALL` to control generation. Flags can be passed either
     in compiled regex or inside the pattern with a ``(?iLmsux)`` group.
 
     Some regular expressions are only partly supported - the underlying
@@ -1022,23 +1057,26 @@ def from_regex(regex):
     mainly includes (positive or negative) lookahead and lookbehind groups.
 
     If you want the generated string to match the whole regex you should use
-    boundary markers. So e.g. ``r"\\A.\\Z"`` will return a single character
-    string, while ``"."`` will return any string, and ``r"\\A.$"`` will return
-    a single character optionally followed by a ``"\\n"``.
+    boundary markers. So e.g. ``r"\A.\Z"`` will return a single character
+    string, while ``"."`` will return any string, and ``r"\A.$"`` will return
+    a single character optionally followed by a ``"\n"``.
+    Alternatively, passing ``fullmatch=True`` will ensure that the whole
+    string is a match, as if you had used the ``\A`` and ``\Z`` markers.
 
     Examples from this strategy shrink towards shorter strings and lower
-    character values.
+    character values, with exact behaviour that may depend on the pattern.
     """
+    check_type(bool, fullmatch, 'fullmatch')
     # TODO: We would like to move this to the top level, but pending some major
     # refactoring it's hard to do without creating circular imports.
     from hypothesis.searchstrategy.regex import regex_strategy
-    return regex_strategy(regex)
+    return regex_strategy(regex, fullmatch)
 
 
 @cacheable
 @defines_strategy_with_reusable_values
 def binary(
-    min_size=None, average_size=None, max_size=None
+    min_size=0, average_size=None, max_size=None
 ):
     # type: (int, int, int) -> SearchStrategy[bytes]
     """Generates the appropriate binary type (str in python 2, bytes in python
@@ -1066,13 +1104,13 @@ def binary(
 @cacheable
 @defines_strategy
 def randoms():
-    # type: () -> SearchStrategy[Random]
-    """Generates instances of Random (actually a Hypothesis specific
-    RandomWithSeed class which displays what it was initially seeded with)
+    # type: () -> SearchStrategy[random.Random]
+    """Generates instances of ``random.Random``, tweaked to show the seed value
+    in the repr for reproducibility.
 
     Examples from this strategy shrink to seeds closer to zero.
     """
-    return RandomStrategy(integers())
+    return integers().map(RandomWithSeed)
 
 
 class RandomSeeder(object):
@@ -1087,25 +1125,30 @@ class RandomSeeder(object):
 class RandomModule(SearchStrategy):
     def do_draw(self, data):
         data.can_reproduce_example_from_repr = False
-        seed = data.draw(integers())
+        seed = data.draw(integers(0, 2 ** 32 - 1))
         state = random.getstate()
         random.seed(seed)
         cleanup(lambda: random.setstate(state))
+        if numpy is not None:  # pragma: no cover
+            npstate = numpy.random.get_state()
+            numpy.random.seed(seed)
+            cleanup(lambda: numpy.random.set_state(npstate))
         return RandomSeeder(seed)
 
 
 @cacheable
 @defines_strategy
 def random_module():
-    """If your code depends on the global random module then you need to use
-    this.
+    """The Hypothesis engine handles PRNG state for the stdlib and Numpy random
+    modules internally, always seeding them to zero and restoring the previous
+    state after the test.
 
-    It will explicitly seed the random module at the start of your test
-    so that tests are reproducible. The value it passes you is an opaque
-    object whose only useful feature is that its repr displays the
-    random seed. It is not itself a random number generator. If you want
-    a random number generator you should use the randoms() strategy
-    which will give you one.
+    If having a fixed seed would unacceptably weaken your tests, and you
+    cannot use a ``random.Random`` instance provided by
+    `:func:`~hypothesis.strategies.randoms`, this strategy calls
+    :func:`python:random.seed` with an arbitrary integer and passes you
+    an opaque object whose repr displays the seed value for debugging.
+    If ``numpy.random`` is available, that state is also managed.
 
     Examples from these strategy shrink to seeds closer to zero.
     """
@@ -1160,7 +1203,7 @@ def builds(
         # Avoid an implementation nightmare juggling tuples and worse things
         raise InvalidArgument('infer was passed as a positional argument to '
                               'builds(), but is only allowed as a keyword arg')
-    required = required_args(target, args, kwargs)
+    required = required_args(target, args, kwargs) or set()
     to_infer = set(k for k, v in kwargs.items() if v is infer)
     if required or to_infer:
         if isclass(target) and attr.has(target):
@@ -1301,8 +1344,7 @@ def from_type(thing):
         return sampled_from(thing)
     # If we know that builds(thing) will fail, give a better error message
     required = required_args(thing)
-    if not any([
-        not required,
+    if required and not any([
         required.issubset(get_type_hints(thing.__init__)),
         attr.has(thing),
         # NamedTuples are weird enough that we need a specific check for them.
@@ -1359,6 +1401,7 @@ def fractions(
             assert max_denominator is not None
             if value is None or value.denominator <= max_denominator:
                 return value, value
+
             p0, q0, p1, q1 = 0, 1, 1, 0
             n, d = value.numerator, value.denominator
             while True:
@@ -1376,8 +1419,22 @@ def fractions(
         # Take the high approximation for min_value and low for max_value
         bounds = (max_denominator, min_value, max_value)
         if min_value is not None:
+            if min_value.denominator > max_denominator:
+                note_deprecation(
+                    'The min_value=%r has a denominator greater than the '
+                    'max_denominator=%r, which will be an error in a future '
+                    'version.'
+                    % (min_value, max_denominator)
+                )
             _, min_value = fraction_bounds(min_value)
         if max_value is not None:
+            if max_value.denominator > max_denominator:
+                note_deprecation(
+                    'The max_value=%r has a denominator greater than the '
+                    'max_denominator=%r, which will be an error in a future '
+                    'version.'
+                    % (max_value, max_denominator)
+                )
             max_value, _ = fraction_bounds(max_value)
 
         if min_value is not None and max_value is not None and \
@@ -1461,7 +1518,7 @@ def decimals(
     places=None,  # type: int
 ):
     # type: (...) -> SearchStrategy[Decimal]
-    """Generates instances of :class:`decimals.Decimal`, which may be:
+    """Generates instances of :class:`python:decimal.Decimal`, which may be:
 
     - A finite rational number, between ``min_value`` and ``max_value``.
     - Not a Number, if ``allow_nan`` is True.  None means "allow NaN, unless
@@ -1572,11 +1629,11 @@ class PermutationStrategy(SearchStrategy):
         self.values = values
 
     def do_draw(self, data):
-        # Reversed Fisher-Yates shuffle. Reverse order so that it shrinks
-        # propertly: This way we prefer things that are lexicographically
-        # closer to the identity.
+        # Reversed Fisher-Yates shuffle: swap each element with itself or with
+        # a later element.  This shrinks i==j for each element, i.e. to no
+        # change.  We don't consider the last element as it's always a no-op.
         result = list(self.values)
-        for i in hrange(len(result)):
+        for i in hrange(len(result) - 1):
             j = integer_range(data, i, len(result) - 1)
             result[i], result[j] = result[j], result[i]
         return result
@@ -1613,8 +1670,8 @@ def datetimes(
     # type: (...) -> SearchStrategy[dt.datetime]
     """A strategy for generating datetimes, which may be timezone-aware.
 
-    This strategy works by drawing a naive datetime between ``min_datetime``
-    and ``max_datetime``, which must both be naive (have no timezone).
+    This strategy works by drawing a naive datetime between ``min_value``
+    and ``max_value``, which must both be naive (have no timezone).
 
     ``timezones`` must be a strategy that generates
     :class:`~python:datetime.tzinfo` objects (or None,
@@ -1647,7 +1704,7 @@ def datetimes(
     # that took aware bounds, but the API and validation is much harder.
     # If you want to generate datetimes between two particular momements in
     # time I suggest (a) just filtering out-of-bounds values; (b) if bounds
-    # are very close, draw a value and subtract it's UTC offset, handling
+    # are very close, draw a value and subtract its UTC offset, handling
     # overflows and nonexistent times; or (c) do something customised to
     # handle datetimes in e.g. a four-microsecond span which is not
     # representable in UTC.  Handling (d), all of the above, leads to a much
@@ -1679,7 +1736,7 @@ def dates(
     min_date=None, max_date=None,
 ):
     # type: (dt.date, dt.date, dt.date, dt.date) -> SearchStrategy[dt.date]
-    """A strategy for dates between ``min_date`` and ``max_date``.
+    """A strategy for dates between ``min_value`` and ``max_value``.
 
     Examples from this strategy shrink towards January 1st 2000.
     """
@@ -1704,7 +1761,7 @@ def times(
     max_time=None,  # type: dt.time
 ):
     # type: (...) -> SearchStrategy[dt.time]
-    """A strategy for times between ``min_time`` and ``max_time``.
+    """A strategy for times between ``min_value`` and ``max_value``.
 
     The ``timezones`` argument is handled as for :py:func:`datetimes`.
 
@@ -1756,13 +1813,7 @@ class CompositeStrategy(SearchStrategy):
         self.kwargs = kwargs
 
     def do_draw(self, data):
-        first_draw = [True]
-
-        def draw(strategy):
-            first_draw[0] = False
-            return data.draw(strategy)
-
-        return self.definition(draw, *self.args, **self.kwargs)
+        return self.definition(data.draw, *self.args, **self.kwargs)
 
     def calc_label(self):
         return self.__label
@@ -1834,8 +1885,8 @@ def complex_numbers(min_magnitude=0, max_magnitude=None,
 
     If you need to generate complex numbers with particular real and
     imaginary parts or relationships between parts, consider using
-    `builds(complex, ...) <hypothesis.strategies.builds>` or
-    `@composite <hypothesis.strategies.composite>` respectively.
+    :func:`builds(complex, ...) <hypothesis.strategies.builds>` or
+    :func:`@composite <hypothesis.strategies.composite>` respectively.
     """
     check_valid_magnitude(min_magnitude, 'min_magnitude')
     check_valid_magnitude(max_magnitude, 'max_magnitude')
@@ -2167,12 +2218,11 @@ def deferred(definition):
 @defines_strategy_with_reusable_values
 def emails():
     """A strategy for generating email addresses as unicode strings. The
-    address format is specific in :rfc:`5322#section-3.4.1`. Values shrink
+    address format is specified in :rfc:`5322#section-3.4.1`. Values shrink
     towards shorter local-parts and host domains.
 
     This strategy is useful for generating "user data" for tests, as
-    mishandling of email addresses is a common source of bugs. Future
-    updates will generate more complicated addresses allowed by the RFC.
+    mishandling of email addresses is a common source of bugs.
     """
     from hypothesis.provisional import domains
     local_chars = string.ascii_letters + string.digits + "!#$%&'*+-/=^_`{|}~"
